@@ -59,11 +59,11 @@ class Backend:
 			return self[key]
 		return default
 
-	def get_binary_list (self):
+	def get_binary_list (self, architecture):
 		"""
 		Return the partial binList associated to this Backend
 		"""
-		return self._dists.get_binary_list()
+		return self._dists.get_binary_list(architecture)
 
 	def get_source_list (self):
 		"""
@@ -71,11 +71,11 @@ class Backend:
 		"""
 		return self._dists.get_source_list()
 
-	def get_full_binary_list (self):
+	def get_full_binary_list (self, architecture):
 		"""
 		Return the full binList associated to this Backend
 		"""
-		return self._dists.get_full_binary_list()
+		return self._dists.get_full_binary_list(architecture)
 
 	def get_full_source_list (self):
 		"""
@@ -137,6 +137,6 @@ class MergeBackend(Backend):
 	def get_mirrors (self):
 		return self._dists.get_mirrors()
 
-	def get_packages_for_mirror(self, mirror):
-		return self._dists.get_packages_for_mirror(mirror)
+	def get_packages_for_mirror(self, mirror, architecture):
+		return self._dists.get_packages_for_mirror(mirror, architecture)
 
