@@ -19,6 +19,7 @@
 # Add a control on md5sum to check wich files has to be updated
 
 import os
+import re
 import time
 
 import apt_pkg
@@ -56,11 +57,11 @@ class Dists:
 		return backends
 
 
-	def _resolve_deps_using_backends(self, requiredPackages, backends):
+	def _resolve_deps_using_backends(self, requiredPackages, backends, architecture):
 		# Resolve our dependencies using pkglists
 		pkglists = []
 		for backend in backends:
-			pkglists.append(backend.get_full_binary_list())
+			pkglists.append(backend.get_full_binary_list(architecture))
 		try:
 			requiredPackages.resolve_depends(pkglists, fastFail=False)
 		except PackageList.BrokenDependencies, exception:
@@ -78,12 +79,12 @@ class MirrorDists(Dists):
 	def __init__(self, backend):
 		Dists.__init__(self, backend)
 
-		# Partial package lists
-		self.__bin = PackageList.PackageList()
+		# Partial package lists per architecture
+		self.__bin = {}
 		self.__source = PackageList.PackageList()
 
-		# Full package lists
-		self.__full_bin	   = PackageList.PackageList()
+		# Full package lists per architecture
+		self.__full_bin	   = {} 
 		self.__full_source = PackageList.PackageList()
 
 	def _fill_files(self):
@@ -127,14 +128,14 @@ class MirrorDists(Dists):
 				files.append(file.split('.gz')[0])
 		return files
 
-	def get_binary_list(self):
-		return self.__bin
+	def get_binary_list(self, architecture):
+		return self.__bin.setdefault(architecture, PackageList.PackageList())
 
 	def get_source_list(self):
 		return self.__source
 
-	def get_full_binary_list(self):
-		return self.__full_bin
+	def get_full_binary_list(self, architecture):
+		return self.__full_bin.setdefault(architecture, PackageList.PackageList())
 
 	def get_full_source_list(self):
 		return self.__full_source
@@ -144,23 +145,28 @@ class MirrorDists(Dists):
 			return
 
 		for file in self.get_index():
+			distribution = file.split(os.path.sep)[1]
+			
 			# Choose object type
 			if os.path.basename(file) == 'Packages':
 				pkg = Package.Package
-				pkglist = self.__full_bin
+				architecture = re.match(r".*/binary-(.*)/.*", file).group(1)
+				pkglist = self.get_full_binary_list(architecture)
 			elif os.path.basename(file) == 'Sources':
 				pkg = Package.SourcePackage
 				pkglist = self.__full_source
+				architecture = "source"
 
 			# Read release file to determine component
 			class ReleaseInfo:
-				def __init__(self, distribution):
+				def __init__(self, distribution, architecture):
 					self.distribution = distribution
+					self.architecture = architecture
 					self.component = None
 				def parse(self, section):
 					self.component = section["Component"]
 
-			releaseInfo = ReleaseInfo(file.split(os.path.sep)[1])
+			releaseInfo = ReleaseInfo(distribution, architecture)
 			release_filename = os.path.join(
 				self._filesystem.base(),
 				os.path.dirname(file),
@@ -189,16 +195,14 @@ class MirrorDists(Dists):
 		self.load()
 		# Apply filter or use as final list
 		if pkgfilter:
-			self.__bin = self.__full_bin.filter(pkgfilter)
+			for architecture in self._backend["architectures"]:
+				self.__bin[architecture] = self.get_full_binary_list(architecture).filter(pkgfilter)
 			self.__source = self.__full_source.filter(pkgfilter)
 		else:
 			self.__bin = self.__full_bin
 			self.__source = self.__full_source
 
 	def resolve(self):
-		# Collect the lists to be used to resolve dependencies
-		pkglists = []
-		backends = []
 		backend_names = [self._backend._name]
 
 		try:
@@ -206,10 +210,12 @@ class MirrorDists(Dists):
 		except Config.InvalidOption:
 			pass # Is possible to don't have this option
 
-		self._resolve_deps_using_backends(
-			self.__bin,
-			self._get_backends_for_names(backend_names),
-		)
+		for architecture in self._backend["architectures"]:			
+			self._resolve_deps_using_backends(
+				self.get_binary_list(architecture),
+				self._get_backends_for_names(backend_names),
+				architecture
+			)
 
 	def process(self):
 		self.filter()
@@ -218,8 +224,9 @@ class MirrorDists(Dists):
 
 	def writeIndexFiles(self):
 		indices = _Indices()
-		for pkg in self.__bin.values():
-			indices.addPackage(pkg, self._backend, pkg.releaseInfo.distribution)
+		for architecture in self._backend["architectures"]:			
+			for pkg in self.get_binary_list(architecture).values():
+				indices.addPackage(pkg, self._backend, pkg.releaseInfo.distribution)
 
 		for dist in self._backend["distributions"]:
 			releaseFile = os.path.join(self._filesystem.base(), "dists", dist, "Release")
@@ -285,9 +292,10 @@ class MergeDists (Dists):
 	def merge(self):
 		indices = _Indices()
 		# Fill package lists
-		for mirror in self.__mirrors:
-			for pkg in self.get_packages_for_mirror(mirror).values():
-				indices.addPackage(pkg, mirror, self._backend['name'])
+		for architecture in self._backend["architectures"]:
+			for mirror in self.__mirrors:
+				for pkg in self.get_packages_for_mirror(mirror, architecture).values():
+					indices.addPackage(pkg, mirror, self._backend['name'])
 
 		# Write package lists and release files
 		release = {"Archive" : self._backend._name}
@@ -309,14 +317,14 @@ class MergeDists (Dists):
 	def get_mirrors (self):
 		return self.__mirrors
 
-	def get_packages_for_mirror(self, mirror):
+	def get_packages_for_mirror(self, mirror, architecture):
 		pkgfilter = []
 		try:
 			pkgfilter = self._backend['filter_%s' % mirror._name]
 		except Config.InvalidOption:
-			return mirror.get_binary_list()
-		packages = mirror.get_binary_list().filter(pkgfilter)
-		self._resolve_deps_using_backends(packages, self.__mirrors)
+			return mirror.get_binary_list(architecture)
+		packages = mirror.get_binary_list(architecture).filter(pkgfilter)
+		self._resolve_deps_using_backends(packages, self.__mirrors, architecture)
 		return packages
 
 
@@ -354,7 +362,8 @@ class _Indices:
 
 		def writeReleaseFields(out, *fieldnames):
 			for fieldname in fieldnames:
-				out.write("%s: %s\n" % (fieldname, release[fieldname]))
+				if fieldname in release:
+					out.write("%s: %s\n" % (fieldname, release[fieldname]))
 
 		for index in self._indexByFilename.values():
 			if not filesystem.exists(index.getDirectory()):
@@ -415,6 +424,7 @@ class _Indices:
 
 			out.write("SHA1:\n")
 			for filename in files:
+				fullpath = "dists/%s/%s" % (dist, filename)
 				out.write(" %s %8d %s\n" % (
 					filesystem.sha1sum(fullpath),
 					filesystem.size(fullpath),
@@ -466,9 +476,8 @@ class _Index:
 def processTagFile(filename, sectionHandler):
 	parse_in = open(filename, "r")
 	try:
-		parse = apt_pkg.ParseTagFile(parse_in)
-		while parse.Step() == 1:
-			sectionHandler(parse.Section)
+		for section in apt_pkg.TagFile(parse_in):
+			sectionHandler(section)
 	finally:
 		parse_in.close()
 
@@ -479,7 +488,8 @@ def tagFileToSectionMapList(filename):
 	def handleSection(section):
 		tags = {}
 		for key in section.keys():
-			tags[key] = section[key]
+			if key:
+				tags[key] = section[key]
 		sections.append(tags)
 
 	processTagFile(filename, handleSection)
